local HenHouse = ...

--------------------------------------------------------------------------
local PV3_HENHOUSE_COMP = {
	TypeName = "PV3_HENHOUSE_COMP",
	ParentType = "COMPONENT",
	Properties = {
		{ Name = "FlagTerrain", Type = "integer", Default = 2 ^ OBJECT_FLAG.TERRAIN:toNumber() },
                { Name = "MinHenHouseSpeed", Type = "float", Default = 0.1 },
                { Name = "MaxHenHouseSpeed", Type = "float", Default = 1.0 },
                { Name = "MaxRandomDistanceMove", Type = "float", Default = 1.0 },
                { Name = "MinHenDeltaX", Type = "float", Default = -1.0 },
                { Name = "MaxHenDeltaX", Type = "float", Default = 1.0 },
                { Name = "MinHenDeltaZ", Type = "float", Default = -1.0 },
                { Name = "MaxHenDeltaZ", Type = "float", Default = 1.0 },
	}
}

--------------------------------------------------------------------------
function PV3_HENHOUSE_COMP:create()
end

--------------------------------------------------------------------------
function PV3_HENHOUSE_COMP:init()
        self._henInitialPosition = self:getOwner():getGlobalPosition()
        HenHouse:log("self._henInitialPosition" .. tostring(self._henInitialPosition))
        self:chooseRandomSequence()
        self:getOwner():setRotationY(self._rotationY )
        HenHouse:log("FlagTerrain = " .. self.FlagTerrain .. " bit TERRAIN = " .. OBJECT_FLAG.TERRAIN:toNumber())
end

--------------------------------------------------------------------------
function PV3_HENHOUSE_COMP:chooseRandomSequence()
        self._henHouseSpeedCurrent = self.MinHenHouseSpeed + math.random() * (self.MaxHenHouseSpeed - self.MinHenHouseSpeed ) 
        self._timer = math.random() * self.MaxRandomDistanceMove / self._henHouseSpeedCurrent
        self._rotationY = (math.random() - 0.5) * 2 * math.pi
--        HenHouse:log("chooseRandomSequence: _rotationY = " .. tostring(self._rotationY))
end

--------------------------------------------------------------------------
function PV3_HENHOUSE_COMP:checkGroundElevation(_henDestination)
    local raycastResult2 = {}
--    HenHouse:log("initial raycastResult2 = " .. tostring(raycastResult2))
     
    -- Raycast from the building position + Y = 100
    -- to building position - y = 100
    -- only on objects with a TERRAIN flag
    local FromPosition2 = { _henDestination[1], _henDestination[2]+100, _henDestination[3] }
    local ToPosition2 = { _henDestination[1], _henDestination[2]-100, _henDestination[3] }
--    HenHouse:log("FromPosition2 = " .. tostring(FromPosition2) .. "ToPosition2 = " .. tostring(ToPosition2))
    self:getLevel():rayCast(FromPosition2, ToPosition2, raycastResult2, self.FlagTerrain)
--    HenHouse:log("final raycastResult2 = RigidBody = " .. tostring(raycastResult2["RigidBody"]) .. " Position = " .. tostring(raycastResult2["Position"]) .. " Normal = " .. tostring(raycastResult2["Normal"]))
    if raycastResult2["RigidBody"] == nil then
         HenHouse:logWarning("HenHouse: Hen NOT on ground")
         return 0
    else
--         HenHouse:log("checkGroundElevation = " .. - _henDestination[2] + raycastResult2["Position"][2])
         return -_henDestination[2] + raycastResult2["Position"][2]
    end
end

--------------------------------------------------------------------------
function PV3_HENHOUSE_COMP:update()
    local dt = self:getLevel():getDeltaTime()
    if self._timer < 0 then
        self:chooseRandomSequence()
        self:getOwner():setRotationY(self._rotationY)
    end
    self._timer = self._timer - dt

        local HenCurrent = self:getOwner():getGlobalPosition()
--        HenHouse:log("HenCurrent = " .. tostring(HenCurrent))
        local HenDestination = { HenCurrent[1] + math.sin(self._rotationY) * dt * self._henHouseSpeedCurrent, 
                                 HenCurrent[2],
                                 HenCurrent[3] + math.cos(self._rotationY) * dt * self._henHouseSpeedCurrent}
--        HenHouse:log("HenDestination = " .. tostring(HenDestination))
        local DeltaX = HenDestination[1] - self._henInitialPosition[1]
--        HenHouse:log("DeltaX = " .. DeltaX)
        if DeltaX < self.MinHenDeltaX then
            DeltaX = self.MinHenDeltaX
            HenDestination[1] = self._henInitialPosition[1] + DeltaX 
        elseif DeltaX > self.MaxHenDeltaX then
            DeltaX = self.MaxHenDeltaX
            HenDestination[1] = self._henInitialPosition[1] + DeltaX 
        end
        local DeltaZ = HenDestination[3] - self._henInitialPosition[3]
        HenHouse:log("DeltaZ = " .. DeltaZ)
        if DeltaZ < self.MinHenDeltaZ then
            DeltaZ = self.MinHenDeltaZ
            HenDestination[3] = self._henInitialPosition[3] + DeltaZ 
        elseif DeltaZ > self.MaxHenDeltaZ then
            DeltaZ = self.MaxHenDeltaZ
            HenDestination[3] = self._henInitialPosition[3] + DeltaZ 
        end
--        HenHouse:log("translate")

        self:getOwner():translate({ HenDestination[1] - HenCurrent[1], self:checkGroundElevation(HenDestination), HenDestination[3] - HenCurrent[3] })

end

--------------------------------------------------------------------------
HenHouse:registerClass(PV3_HENHOUSE_COMP)
